unit DBGridFind;

//          

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, DBGrids;

type
  TDBGridFind = class(TCommonDialog)

  private
    FDBGrid: TDBGrid;
  protected
    SearchFieldName: String;
    FindDialog: TFindDialog;
    procedure FindDialogOnFind(Sender: TObject);
    procedure FindDialogOnClose(Sender: TObject);
    procedure FindDialogOnShow(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean; override;
  published
    property DBGrid: TDBGrid read FDBGrid write FDBGrid default nil;
  end;

procedure Register;

implementation

constructor TDBGridFind.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FindDialog:=TFindDialog.Create(Self);
with FindDialog do
  begin
    OnFind:=FindDialogOnFind;
    OnClose:=FindDialogOnClose;
    OnShow:=FindDialogOnShow;
  end;
end;

destructor TDBGridFind.Destroy;
begin
FindDialog.Destroy;
inherited Destroy;
end;

procedure TDBGridFind.FindDialogOnClose(Sender: TObject);
begin
if Assigned(OnClose) then OnClose(Sender);
end;

procedure TDBGridFind.FindDialogOnShow(Sender: TObject);
begin
if Assigned(OnShow) then OnShow(Sender);
end;

function TDBGridFind.Execute: Boolean;
var VFindText: String;
begin
Result:=Assigned(DBGrid) and Assigned(DBGrid.DataSource)
  and Assigned(DBGrid.DataSource.DataSet)
  and DBGrid.DataSource.DataSet.Active;
if Result then
  begin
   FindDialog.Ctl3d:=Ctl3d;
   with DBGrid do
     begin
       SearchFieldName:=SelectedField.FieldName;
       with FindDialog do
         with DbGrid.DataSource.DataSet do
            begin
              with FieldByName(SearchFieldName) do
                if Assigned(OnGetText) then
                 begin
                  OnGetText(DataSet.FieldByName(SearchFieldName),VFindText, True);
                  FindText:=VFindText;
                 end
                  else FindText:=AsString;
              FindDialog.Execute;
            end;
     end;
  end;
end;

procedure TDBGridFind.FindDialogOnFind(Sender: TObject);
var RecordNo: Integer; SearchText: String;

  function CmpField(Value1, Value2: String;
       MatchCase, WholeWord: boolean): boolean;
  begin
    if not MatchCase then
      begin
        Value1:=AnsiLowerCase(Value1); Value2:=AnsiLowerCase(Value2);
      end;
    if WholeWord then Result:=Value1 = Value2
      else
         Result:=Pos(Value2, Value1) <> 0;
  end;

begin
try
with FindDialog do with DBGrid.DataSource.DataSet do
  begin
    DisableControls;
    RecordNo:=DBGrid.DataSource.DataSet.RecNo;
    repeat
      if frDown in Options then
        begin
          Next; if EOF then First;
        end
          else
            begin
              Prior; if BOF then Last;
            end;
      if RecordNo = DBGrid.DataSource.DataSet.RecNo  then
        begin
          Application.MessageBox('  !','',MB_ICONERROR);
          Break;
        end;
      with FieldByName(SearchFieldName) do
        if Assigned(OnGetText) then
          OnGetText(DataSet.FieldByName(SearchFieldName),SearchText, True)
            else SearchText:=AsString;
    until CmpField(SearchText, FindText, frMatchCase in Options,
       frWholeWord in Options);
    try
      DisableControls;
      DBGrid.Repaint;
    finally EnableControls; end;
  end;
finally DBGrid.DataSource.DataSet.EnableControls; end;
end;

procedure Register;
begin
  RegisterComponents('Alien', [TDBGridFind]);
end;

initialization

finalization
end.

