unit AvtoSave;

//     ;

interface

uses
  Windows, Messages, SysUtils, Classes, ExtCtrls, Graphics, Controls,
  Forms, Dialogs, INIFiles;

type
  TErrorLoadFileEvent = procedure (Sender: TObject;
                     FileName: String; ErrorCode: Integer) of object;
  TErrorSaveFileEvent = procedure (Sender: TObject; FileName: String;
                     ErrorCode: Integer; var SaveError: boolean) of object;

  TSaveFileEvent = procedure (Sender: TObject; var NeedSave: Boolean) of object;

  TAvtoSave = class(TComponent)
  private
    Timer: TTimer;
    FSaveOnThread: boolean;
    FININame, FINISection, FINIPath: String;
    FSaving, FActive, FSetNL, FResetNS: boolean;
    FSaveString: TStrings;
    FSaveTime, FTimerCounter: Integer;
    FOnErrorSave: TErrorSaveFileEvent;
    FOnErrorLoad: TErrorLoadFileEvent;
    FOnBeginSave: TSaveFileEvent;
    FOnEndSave, FOnEndLoad, FOnBeginLoad, FOnDestroy: TNotifyEvent;
    FSaveFileDir, FFileDir: String;
    function CorrectPath(Value: String): String;
    procedure SetActive(AValue: boolean);
    procedure SetSaveTime(AValue: Integer);
    procedure SetSaveString(AValue: TStrings);
    procedure SetSaveFileDir(AValue: String);
    procedure SetFileDir(AValue: String);
    procedure SetSaveOk(AValue: boolean);
    function GetSaveOk: boolean;
    procedure SetNeedLoad(AValue: boolean);
    function GetNeedLoad: boolean;
    function GetIniFileName: String;
    procedure TimerOn(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Save; //   
    procedure Load; //   (   )
    function GetIniFile: TINIFile; //     
                     //   ( )
    property INIFileName: String read GetIniFileName;
                     //     .
    property SaveOk: boolean read GetSaveOk write SetSaveOk;
                     //    (  Load)
    property NeedLoad: boolean read GetNeedLoad write SetNeedLoad;
  published
         //  True  .
    property Active: Boolean read FActive write SetActive default False;
         //   ,    
    property SaveOnThread:Boolean read FSaveOnThread write FSaveOnThread default False;
         //     NeedLoad
    property SetNeedLoadOnSave: Boolean read FSetNL write FSetNL;
         //     NeedLoad
    property ResetNeedSaveOnDestroy: Boolean read FResetNS write FResetNS;
         //   INI 
    property INIPath: String read FINIPath write FINIPath;
    property ININame: String read FININame write FININame;
    property INISection: String read FINISection write FINISection;
       //      
    property Saving: boolean read FSaving;
       //   
    property SaveFiles: TStrings read FSaveString write SetSaveString;
      //  ...
    property SaveTime: Integer read FSaveTime write SetSaveTime default 0;
      //  ...
    property SaveFileDir: String read FSaveFileDir write SetSaveFileDir;
      //  ...
    property FileDir: String read FFileDir write SetFileDir;
    property OnBeginSave: TSaveFileEvent read FOnBeginSave write FOnBeginSave;
    property OnEndSave: TNotifyEvent read FOnEndSave write FOnEndSave;
    property OnBeginLoad: TNotifyEvent read FOnBeginLoad write FOnBeginLoad;
    property OnEndLoad: TNotifyEvent read FOnEndLoad write FOnEndLoad;
    property OnErrorLoad: TErrorLoadFileEvent
                   read FOnErrorLoad write FOnErrorLoad;
    property OnErrorSave: TErrorSaveFileEvent
                   read FOnErrorSave write FOnErrorSave;
    property OnDestroy: TNotifyEvent read FOnDestroy write FOnDestroy;
  end;

  TAutoSaveThread = class(TThread)
  private
    AvtoSave: TAvtoSave;
  protected
    procedure Execute; override;
  end;

procedure Register;

implementation

var SearchRec: TSearchRec; INIFile: TINIFile;

function DirExists(Dir: String): boolean;
var Tmp: String;
begin
Result:=False; Tmp:=Dir;
if Length(Tmp) > 3 then
  begin
    if Tmp[Length(Tmp)] = '\' then Delete(Tmp, Length(Tmp), 1);
    Result:=FindFirst(Tmp, faDirectory, SearchRec) = 0;
  end;
end;

constructor TAvtoSave.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FSaveString:=TStringList.Create;
Timer:=TTimer.Create(nil); Timer.OnTimer:=TimerOn;
FININame:=''; SaveTime:=0; SaveOnThread:=False;
FSaving:=False; FSetNL:=False; FResetNS:=False;
end;

destructor TAvtoSave.Destroy;
begin // Destroy   
while Saving do Application.ProcessMessages;
if ResetNeedSaveOnDestroy then NeedLoad:=False;
if Assigned(OnDestroy) then OnDestroy(Self);
Timer.Destroy;
FSaveString.Destroy;
inherited Destroy;
end;

function TAvtoSave.CorrectPath(Value: String): String;
begin
if (Length(Value) <> 0) and (Value[Length(Value)] = '\') then
  Delete(Value, Length(Value), 1);
Result:=Value;
end;

function TAvtoSave.GetIniFileName: String;
begin
if (INISection <> '') and (ININame <> '') then
  try
    Result:=CorrectPath(INIPath);
    if DirExists(Result) then
        Result:=Result + '\' + ININame
      else Result:=ININame;
  except Result:=''; end
end;

function TAvtoSave.GetIniFile: TINIFile;
begin
 if INIFileName <> '' then
   try
     Result:=TINIFile.Create(IniFileName);
   except Result:=nil; end
     else Result:=nil;
end;

procedure TAvtoSave.SetActive(AValue: boolean);
begin
FActive:=AValue; SaveTime:=SaveTime;
end;

procedure TAvtoSave.SetSaveOk(AValue: boolean);
begin
INIFile:=GetIniFile;
if Assigned(INIFile) then
  with INIFile do
    try
      WriteBool(INISection, Self.Name + '_SaveOk', AValue);
    finally INIFile.Destroy end;
end;

function TAvtoSave.GetSaveOk: boolean;
begin
INIFile:=GetIniFile; Result:=False;
if Assigned(INIFile) then
  with INIFile do
    try
      Result:=ReadBool(INISection, Self.Name + '_SaveOk', False);
    finally INIFile.Destroy end;
end;

procedure TAvtoSave.SetNeedLoad(AValue: boolean);
begin
INIFile:=GetIniFile;
if Assigned(INIFile) then
  with INIFile do
    try
      WriteBool(INISection, Self.Name + '_NeedLoad', AValue);
    finally INIFile.Destroy end;
end;

function TAvtoSave.GetNeedLoad: boolean;
begin
INIFile:=GetIniFile; Result:=False;
if Assigned(INIFile) then
  with INIFile do
    try
      Result:=ReadBool(INISection, Self.Name + '_NeedLoad', False);
    finally INIFile.Destroy end;
end;

procedure TAvtoSave.TimerOn(Sender: TObject);
begin
if FTimerCounter < FSaveTime then
  Inc(FTimerCounter)
    else
      begin Inc(FTimerCounter); Save; FTimerCounter:=0; end;

end;

procedure TAvtoSave.SetSaveTime(AValue: Integer);
begin
FSaveTime:=AValue; FTimerCounter:=0;
Timer.Enabled:=Active and (AValue <> 0);
end;

procedure TAvtoSave.SetSaveString(AValue: TStrings);
begin
FSaveString.Assign(AValue);
end;

procedure TAvtoSave.SetSaveFileDir(AValue: String);
begin
FSaveFileDir:=CorrectPath(AValue);
end;

procedure TAvtoSave.SetFileDir(AValue: String);
begin
FFileDir:=CorrectPath(AValue);
end;

procedure TAvtoSave.Save;
var NeedSave, SavingOk, SaveError: Boolean; Cycle: Integer;
begin
NeedSave:=True;
if Assigned(OnBeginSave) then OnBeginSave(Self, NeedSave);
if (not NeedSave) or Saving then Exit;
if SaveOnThread then
  with TAutoSaveThread.Create(True) do
     begin
       AvtoSave:=Self; Priority:=tpNormal;
       Resume;
     end
       else
         begin
           if Saving then Exit;
           if SetNeedLoadOnSave then NeedLoad:=True;
           FSaving:=True; SaveOk:=False; SavingOk:=True;
           with SaveFiles do
           for Cycle:=0 to Count-1 do
             try
               SaveError:=True;
               if (not CopyFile(PChar(FileDir + '\' + Strings[Cycle]),
                   PChar(SaveFileDir + '\' + Strings[Cycle]), False)) then
                      begin
                        if Assigned(OnErrorSave) then
                             OnErrorSave(Self, Strings[Cycle],
                                  GetLastError, SaveError);
                             SavingOk:=SavingOk and (not SaveError);
                      end;
             except
               if Assigned(OnErrorSave) then
                 OnErrorSave(Self, Strings[Cycle], GetLastError, SaveError);
                  SavingOk:=SavingOk and (not SaveError);
             end;
           SaveOk:=SavingOk; FSaving:=False;
           if Assigned(OnEndSave) then OnEndSave(Self);
         end;
end;

procedure TAvtoSave.Load;
var Cycle: Integer;
begin
if (not SaveOk) and (Application.MessageBox(
   '   c .' + #10 + #13 +
   '  ?', '', MB_IconError + MB_YESNO) = ID_NO)
     then Exit;
while Saving do
    Application.ProcessMessages;
if Assigned(OnBeginLoad) then OnBeginLoad(Self);
with SaveFiles do
  for Cycle:=0 to Count-1 do
    try
      if (not CopyFile(PChar(SaveFileDir + '\' + Strings[Cycle]),
        PChar(FileDir + '\' + Strings[Cycle]), False)) and Assigned(OnErrorLoad)
         then OnErrorLoad(Self, Strings[Cycle], GetLastError);
    except
      if Assigned(OnErrorLoad) then OnErrorLoad(Self, Strings[Cycle], IOResult);
    end;
NeedLoad:=False;
if Assigned(OnEndLoad) then OnEndLoad(Self);
end;

procedure TAutoSaveThread.Execute;
var Cycle: Integer; SavingOk, SaveError: Boolean;
begin
with AvtoSave do
  begin
    if Saving then Exit;
    if SetNeedLoadOnSave then NeedLoad:=True;
    FSaving:=True;
    SaveOk:=False; SavingOk:=True;
    with SaveFiles do
      for Cycle:=0 to Count-1 do
        try
          SaveError:=True;
          if (not CopyFile(PChar(FileDir + '\' + Strings[Cycle]),
            PChar(SaveFileDir + '\' + Strings[Cycle]), False)) then
                begin
                  if Assigned(OnErrorSave) then
                     OnErrorSave(AvtoSave, Strings[Cycle],
                                  GetLastError, SaveError);
                  SavingOk:=SavingOk and (not SaveError);
                end;
        except
          if Assigned(OnErrorSave) then
            OnErrorSave(AvtoSave, Strings[Cycle], GetLastError, SaveError);
          SavingOk:=SavingOk and (not SaveError);
        end;
    SaveOk:=SavingOk; FSaving:=False;
    if Assigned(OnEndSave) then OnEndSave(Self);
    Self.Destroy;
  end;
end;

procedure Register;
begin
  RegisterComponents('Alien', [TAvtoSave]);
end;

end.
